@@ -71,11 +71,13 @@
 #' \code{214} = rotated Tawn type 2 copula (180 degrees) \cr
 #' \code{224} = rotated Tawn type 2 copula (90 degrees) \cr
 #' \code{234} = rotated Tawn type 2 copula (270 degrees) \cr
-#' @param method Character indicating the estimation method: either maximum
+#' @param method indicates the estimation method: either maximum
 #' likelihood estimation (\code{method = "mle"}; default) or inversion of
-#' Kendall's tau (\code{method = "itau"}).\cr For \code{method = "itau"} only
-#' one parameter bivariate copula families can be used (\code{family =
-#' 1,3,4,5,6,13,14,16,23,24,26,33,34} or \code{36}).
+#' Kendall's tau (\code{method = "itau"}). For \code{method = "itau"} only
+#' one parameter families and the Student t copula can be used (\code{family =
+#' 1,2,3,4,5,6,13,14,16,23,24,26,33,34} or \code{36}). For the t-copula,
+#' \code{par2} is found by a crude profile likelihood optimization over the
+#' interval (2, 10].
 #' @param se Logical; whether standard error(s) of parameter estimates is/are
 #' estimated (default: \code{se = FALSE}).
 #' @param max.df Numeric; upper bound for the estimation of the degrees of
@@ -185,7 +187,7 @@ BiCopEst <- function(u1, u2, family, method = "mle", se = FALSE, max.df = 30,
             u <- cbind(u1, u2)
             v <- matrix(0, n, p * (p - 1)/2)
 
-            if (family == 1)
+            if (family %in% 1:2)
                 tauder <- function(x) {
                     2/(pi * sqrt(1 - x^2))
                 } else if (family %in% c(3, 13, 23, 33)) {
@@ -228,7 +230,7 @@ BiCopEst <- function(u1, u2, family, method = "mle", se = FALSE, max.df = 30,
 
             if (family == 0) {
                 D <- 0
-            } else if (family %in% c(1, 3, 4, 5, 6, 13, 14, 16, 41, 51)) {
+            } else if (family %in% c(1, 2, 3, 4, 5, 6, 13, 14, 16, 41, 51)) {
                 D <- 1/tauder(theta)
             } else if (family %in% c(23, 33, 24, 34, 26, 36, 61, 71)) {
                 D <- 1/tauder(-theta)
@@ -237,6 +239,19 @@ BiCopEst <- function(u1, u2, family, method = "mle", se = FALSE, max.df = 30,
 
             se1 <- as.numeric(sqrt(16/n * var(v %*% D)))
         }  # end if (se == TRUE)
+        if (family == 2) {
+            opt <- MLE_intern(cbind(u1, u2),
+                              c(theta, 6),
+                              family = family,
+                              se,
+                              max.df,
+                              max.BB,
+                              weights,
+                              cor.fixed = TRUE)
+            theta <- c(theta, opt$par[2])
+            if (se)
+                se1 <- c(se1, opt$se)
+        }
     }  # end if (method == "itau")
 
     ## MLE ------------------------------------------
@@ -458,36 +473,37 @@ BiCopEst.intern <- function(u1, u2, family, method = "mle", se = TRUE, max.df =
             u <- cbind(u1, u2)
             v <- matrix(0, n, p * (p - 1)/2)
 
-            if (family == 1)
+            if (family %in% 1:2) {
                 tauder <- function(x) {
                     2/(pi * sqrt(1 - x^2))
-                } else if (family %in% c(3, 13, 23, 33)) {
-                    tauder <- function(x) 2 * (2 + x)^(-2)
-                } else if (family %in% c(4, 14, 24, 34)) {
-                    tauder <- function(x) x^(-2)
-                } else if (family == 5) {
-                    f <- function(x) x/(exp(x) - 1)
-                    tauder <- function(x) {
-                        lwr <- 0 + .Machine$double.eps^0.5
-                        intgrl <- integrate(f,
-                                            lower = lwr,
-                                            upper = x)$value
-                        4/x^2 - 8/x^3 * intgrl + 4/(x * (exp(x) - 1))
-                    }
-                } else if (family %in% c(6, 16, 26, 36)) {
-                    tauder <- function(x) {
-                        euler <- 0.577215664901533
-                        -((-2 + 2 * euler + 2 * log(2) + digamma(1/x) +
-                               digamma(1/2 * (2 + x)/x) + x)/(-2 + x)^2) +
-                            ((-trigamma(1/x)/x^2 + trigamma(1/2 * (2 + x)/x) *
-                                  (1/(2 + x) - (2 + x)/(2 * x^2)) + 1)/(-2 + x))
-                    }
-                } else if (family %in% c(41, 51, 61, 71)) {
-                    tauder <- function(x) {
-                        2 * sqrt(pi) * gamma(0.5 + x) *
-                            (digamma(1 + x) - digamma(0.5 + x))/gamma(1 + x)
-                    }
                 }
+            } else if (family %in% c(3, 13, 23, 33)) {
+                tauder <- function(x) 2 * (2 + x)^(-2)
+            } else if (family %in% c(4, 14, 24, 34)) {
+                tauder <- function(x) x^(-2)
+            } else if (family == 5) {
+                f <- function(x) x/(exp(x) - 1)
+                tauder <- function(x) {
+                    lwr <- 0 + .Machine$double.eps^0.5
+                    intgrl <- integrate(f,
+                                        lower = lwr,
+                                        upper = x)$value
+                    4/x^2 - 8/x^3 * intgrl + 4/(x * (exp(x) - 1))
+                }
+            } else if (family %in% c(6, 16, 26, 36)) {
+                tauder <- function(x) {
+                    euler <- 0.577215664901533
+                    -((-2 + 2 * euler + 2 * log(2) + digamma(1/x) +
+                           digamma(1/2 * (2 + x)/x) + x)/(-2 + x)^2) +
+                        ((-trigamma(1/x)/x^2 + trigamma(1/2 * (2 + x)/x) *
+                              (1/(2 + x) - (2 + x)/(2 * x^2)) + 1)/(-2 + x))
+                }
+            } else if (family %in% c(41, 51, 61, 71)) {
+                tauder <- function(x) {
+                    2 * sqrt(pi) * gamma(0.5 + x) *
+                        (digamma(1 + x) - digamma(0.5 + x))/gamma(1 + x)
+                }
+            }
 
             l <- 1
             for (j in 1:(p - 1)) {
@@ -501,7 +517,7 @@ BiCopEst.intern <- function(u1, u2, family, method = "mle", se = TRUE, max.df =
 
             if (family == 0) {
                 D <- 0
-            } else if (family %in% c(1, 3, 4, 5, 6, 13, 14, 16, 41, 51)) {
+            } else if (family %in% c(1, 2, 3, 4, 5, 6, 13, 14, 16, 41, 51)) {
                 D <- 1/tauder(theta)
             } else if (family %in% c(23, 33, 24, 34, 26, 36, 61, 71)) {
                 D <- 1/tauder(-theta)
@@ -510,6 +526,19 @@ BiCopEst.intern <- function(u1, u2, family, method = "mle", se = TRUE, max.df =
 
             se1 <- as.numeric(sqrt(16/n * var(v %*% D)))
         }  # end if (se == TRUE)
+        if (family == 2) {
+            opt <- MLE_intern(cbind(u1, u2),
+                              c(theta, 6),
+                              family = family,
+                              se,
+                              max.df,
+                              max.BB,
+                              weights,
+                              cor.fixed = TRUE)
+            theta <- c(theta, opt$par[2])
+            if (se)
+                se1 <- c(se1, opt$se)
+        }
     }  # end if (method == "itau")
 
     ## MLE ------------------------------------------
@@ -892,28 +921,28 @@ MLE_intern <- function(data, start.parm, family, se = FALSE, max.df = 30,
                                       fn = t_LL,
                                       gr = gr_LL,
                                       method = "L-BFGS-B",
-                                      control = list(fnscale = -1, maxit = 500),
+                                      control = list(fnscale = -1, maxit = 10),
                                       hessian = TRUE,
                                       lower = 2.0001,
-                                      upper = max.df)
+                                      upper = 10)
                 } else {
                     optimout <- optim(par = start.parm[2],
                                       fn = t_LL,
                                       method = "L-BFGS-B",
-                                      control = list(fnscale = -1, maxit = 500),
+                                      control = list(fnscale = -1, maxit = 10),
                                       hessian = TRUE,
                                       lower = 2.0001,
-                                      upper = max.df)
+                                      upper = 10)
                 }
             } else {
                 optimout <- optimize(f = t_LL,
                                      maximum = TRUE,
-                                     interval = c(2.0001, max.df))
+                                     interval = c(2.0001, 10),
+                                     tol = 1)
                 optimout$par <- optimout$maximum
                 optimout$value <- optimout$objective
             }
             optimout$par <- c(0, optimout$par)
-
         }
 
     } else {
