@@ -1,8 +1,8 @@
 #' Simulation from an R-Vine Copula Model
-#' 
+#'
 #' This function simulates from a given R-vine copula model.
-#' 
-#' 
+#'
+#'
 #' @param N Number of d-dimensional observations to simulate.
 #' @param RVM An \code{\link{RVineMatrix}} object containing the information of
 #' the R-vine copula model.
@@ -16,7 +16,7 @@
 #' (2013). Selecting and estimating regular vine copulae and application to
 #' financial returns. Computational Statistics & Data Analysis, 59 (1), 52-69.
 #' @examples
-#' 
+#'
 #' # define 5-dimensional R-vine tree structure matrix
 #' Matrix <- c(5, 2, 3, 1, 4,
 #'             0, 2, 3, 4, 1,
@@ -24,7 +24,7 @@
 #'             0, 0, 0, 4, 1,
 #'             0, 0, 0, 0, 1)
 #' Matrix <- matrix(Matrix, 5, 5)
-#' 
+#'
 #' # define R-vine pair-copula family matrix
 #' family <- c(0, 1, 3, 4, 4,
 #'             0, 0, 3, 4, 1,
@@ -32,7 +32,7 @@
 #'             0, 0, 0, 0, 3,
 #'             0, 0, 0, 0, 0)
 #' family <- matrix(family, 5, 5)
-#' 
+#'
 #' # define R-vine pair-copula parameter matrix
 #' par <- c(0, 0.2, 0.9, 1.5, 3.9,
 #'          0, 0, 1.1, 1.6, 0.9,
@@ -40,40 +40,40 @@
 #'          0, 0, 0, 0, 4.8,
 #'          0, 0, 0, 0, 0)
 #' par <- matrix(par, 5, 5)
-#' 
+#'
 #' # define second R-vine pair-copula parameter matrix
 #' par2 <- matrix(0, 5, 5)
-#' 
+#'
 #' # define RVineMatrix object
 #' RVM <- RVineMatrix(Matrix = Matrix, family = family,
 #'                    par = par, par2 = par2,
 #'                    names = c("V1", "V2", "V3", "V4", "V5"))
-#' 
+#'
 #' # simulate a sample of size 300 from the R-vine copula model
 #' set.seed(123)
 #' simdata <- RVineSim(300, RVM)
-#' 
+#'
 #' @export RVineSim
 RVineSim <- function(N, RVM, U = NULL) {
-    
+
     ## sanity checks
     stopifnot(N >= 1)
-    if (!is(RVM, "RVineMatrix")) 
+    if (!is(RVM, "RVineMatrix"))
         stop("'RVM' has to be an RVineMatrix object.")
-    
+
     ## reorder matrix and U (if provided)
     n <- dim(RVM)
     o <- diag(RVM$Matrix)
     RVM <- normalizeRVineMatrix(RVM)
     takeU <- !is.null(U)
     if (takeU) {
-        if (!is.matrix(U)) 
+        if (!is.matrix(U))
             U <- rbind(U, deparse.level = 0L)
-        if ((d <- ncol(U)) < 2) 
+        if ((d <- ncol(U)) < 2)
             stop("U should be at least bivariate")  # should be an (N, n) matrix
         U <- U[, rev(o)]
     }
-    
+
     ## create objects for C-call
     matri <- as.vector(RVM$Matrix)
     w1 <- as.vector(RVM$family)
@@ -88,24 +88,24 @@ RVineSim <- function(N, RVM, U = NULL) {
     maxmat[is.na(maxmat)] <- 0
     conindirect[is.na(conindirect)] <- 0
     tmp <- rep(0, n * N)
-    
+
     ## simulate R-Vine
-    tmp <- .C("SimulateRVine", 
+    tmp <- .C("SimulateRVine",
               as.integer(N),
               as.integer(n),
-              as.integer(w1), 
+              as.integer(w1),
               as.integer(maxmat),
               as.integer(matri),
-              as.integer(conindirect), 
-              as.double(th), 
+              as.integer(conindirect),
+              as.double(th),
               as.double(th2),
-              as.double(tmp), 
+              as.double(tmp),
               as.double(U),
-              as.integer(takeU), 
+              as.integer(takeU),
               PACKAGE = "VineCopula")[[9]]
-    
-    ## store results, bring back to initial order and return 
-    out <- matrix(tmp, ncol = n)
+
+    ## store results, bring back to initial order and return
+    out <- matrix(tmp, ncol = n, byrow = TRUE)
     if (!is.null(RVM$names)) {
         colnames(out) <- RVM$names
     }
@@ -116,13 +116,14 @@ RVineSim <- function(N, RVM, U = NULL) {
 
 transform <- function(M) {
     n <- dim(M)[1]
-    
+
     M.new <- matrix(rep(0, n * n), n, n)
     for (i in 1:n) {
         for (j in 1:i) {
             M.new[(n - i + 1), (n - j + 1)] <- M[i, j]
         }
     }
-    
+
     return(M.new)
 }
+
